;4840 project, may 2004

; GLOBALS
	list	p=16f77
	include	<p16f77.inc>

	__CONFIG  _CP_OFF & _WDT_ON & _BODEN_ON & _PWRTE_ON & _XT_OSC

c1	equ	0x20	; define ASM alias
tmp	equ	0x21
tmp2	equ	0x22
tmp3	equ	0x23
adc0	equ	0x30
adc1	equ	0x31
adc2	equ	0x32
tmr0	equ	0x33
tmr1	equ	0x34
adc5	equ	0x35
adc6	equ	0x36
adc7	equ	0x37

	org	0x00	; set program origin
reset	GOTO	start

	org	0x04
start
;DISABLE INTERRUPTS
	CLRF INTCON 

;INIT UART, ASSUME Fosc=4 MHZ
	BSF	STATUS,RP0	;BANK1
	MOVLW	D'25'	; 9600 BAUD WITH 4MHZ CLK
	MOVWF	SPBRG	;BAUD RATE GENERATOR REGISTER
	MOVLW	0x24    ;'BRGH'=1 FOR HIGH SPEED
	MOVWF	TXSTA	;XMIT STATUS REG
	BCF	STATUS,RP0	;BANK0
	MOVLW	B'10000000'	;'SPEN'
	MOVWF	RCSTA	;RCSTATUS REG

;ADC INIT
	BSF	STATUS,RP0	;BANK1
	CLRF	ADCON1	;INIT ALL 8 ADC PINS
			;WITH VDD FOR VREF
	BCF	STATUS,RP0	;BANK0
	MOVLW	B'01000001'
	MOVWF	ADCON0	;CHANNEL0, ADC ON, FOSC/8

;PORTB INIT,
;this I/O port was used for debugginng output
	BSF	STATUS,RP0	;BANK1
	CLRF	TRISB	;SET ALL 8BITS FOR OUTPUT
	BCF	STATUS,RP0	;BANK0
	CLRF	PORTB	;CLEAR PORTB TO INIT

;TIMER0 INIT
	BSF	STATUS,RP0	;BANK1
	BCF	STATUS,RP0	;BANK0
	CLRF	TMR0	; CLEAR TIMER

loop	
;This the the main program loop.  This loop consists of a 
;polling stage followed by a transmission stage.

;here we set the indirect address register to point to the
;beginning of the acquired data array
	MOVLW	adc0
	MOVWF	FSR	;LOAD FSR W/ADC0 ADDR

adc_poll
;this is the start of the polling stage
;The following code is executed for each of the 8 ADC channels
;Each iteration, the data from adcY is stored in mem[0x3Y].
	;DELAY FOR ACQUISITION
	CLRF	tmp
HERE1	INCFSZ	tmp,1
	GOTO	HERE1	;;;

	;START CONVERSION	
	BSF	ADCON0,GO
wait1	BTFSC	ADCON0,GO
	GOTO	wait1
	MOVF	ADRES,0
	MOVWF	INDF	;POINTS TO DATA ARRAY

	BTFSS	ADCON0,3
	GOTO	adc_not_done
	BTFSS	ADCON0,4
	GOTO	adc_not_done
	BTFSS	ADCON0,5
	GOTO	adc_not_done
	MOVLW	B'11000111'
	ANDWF	ADCON0,1  ;CLEAR CHANNEL BITS
	GOTO	adc_done
 
adc_not_done
	MOVF	ADCON0,0	;INCREMENT ADC
	ADDLW	B'00001000'	; CHANNEL
	MOVWF	ADCON0		;

	INCF	FSR,1	
	GOTO	adc_poll	

adc_done
	INCF	c1,1	;INCREMENT c1
	
	;OUTPUT TO PORTB FOR DEBUGGING
	MOVF	c1,0
	MOVWF	PORTB

timer_poll
;Timer polling is trivial, just copy from one 
;register to another.
;Timer0 is reset but Timer1 is not
	MOVF	TMR0,0
	MOVWF	tmr0
	CLRF	TMR0

	BCF	T1CON, TMR1ON 	;turn off timer
	MOVF	TMR1L,0
	MOVWF	tmr1
	CLRF	TMR1L
	CLRF	TMR1H
	MOVLW	B'00111110'	;1:8 PRESCALAR, EXTERNAL CLOCK, synchronous
	MOVWF	T1CON		;
	BSF	T1CON, TMR1ON 	;turn on timer


; XMIT PHASE
;Here the data transmission stage begins.  All the acquired
;data was stored in mem[0x30-0x37].  Here we simply iterate
;through this data, again using the indirect address register.
;
;Before loading the UART transmission register TXREG we wait
;for the "ready" bit to be set, TXIF.  When this bit is set,
;we know that the previous byte of data has been sent from
;TXREG to the UART.

	;XMIT HEADER BYTE 0x0D
wait4	BTFSS	PIR1,TXIF
	GOTO wait4	;;;
	MOVLW	0x0D
	MOVWF	TXREG		;LOAD W into TXREG

	;XMIT ADC DATA
	CLRF	tmp2
	MOVLW	adc0
	MOVWF	FSR	;LOAD FSR W/ADC0 ADDR
array_xmit
	SWAPF	INDF,1
	MOVF	INDF,0
	ANDLW	B'00001111'	
	ADDLW	0x30	;ASCII VALUE OF '0'
	MOVWF	tmp
	BTFSS	tmp,3
	GOTO	wait3
	ADDLW	0x07	;OFFSET BETWEEN '9' AND 'A'
	BTFSC	tmp,1	;UNDO IF '8' OR '9'
	GOTO	wait3	; .
	BTFSC	tmp,2	; .
	GOTO	wait3	; .
	ADDLW	0xF9	; . SUBTRACT 0x07

wait3	BTFSS	PIR1,TXIF	;'TXIF'
	GOTO	wait3	;;;
	MOVWF	TXREG		;LOAD W into TXREG

	INCF	tmp2,1
	BTFSC	tmp2,0		;IF FIRST ITERATION
	GOTO	array_xmit

	INCF	FSR,1
	BTFSS	FSR,3	;SET AFTER 8 ITERATIONS
	GOTO	array_xmit	

	GOTO 	loop
	end
