#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/select.h>
#include <fcntl.h>
#include <termios.h>
#include <string.h>
#include <errno.h>

#define DEBUG 0   
#define BSIZE 1700       /*this is actually the last index in buffer*/
#define PSIZE 17         /*packet size (includeing \r in header)*/
#define DATINPACK 8      /*number of data lines in packet*/
#define VPI .0195        /*voltage per incriment*/
#define RPMCONV 282.353  /*conversion rate for v -> RPM*/
#define TPSOFF 0.51   /*voltage offset for throttle*/
#define TPSCONV 0.25     /*throttle conversion rate*/
#define MAPCONV 2.72     /*manifold absolute pressure conversion rate*/
#define MAPOFF 1.51      /*V offset for manifold absolute pressure*/ 
#define PRESCALER 8      /*Prescaling used to encode fuel consumption*/
#define T1FREQ 100000    /*timer one clock freq*/
#define FLOWRT 0.004505  /*flowrate for timer 1*/
#define OUTPUTCNTR 10	 /*used to printf every 10th packet*/

/*
 * Code to open and read RS232 is modified code from RS232.c written by
 * Cristian Petrus Soviani
 */

int main () {

  /*variables for opening/reading RS232*/
  int fd, nbrx, nbtx, ptx, prx, nb;
  fd_set rfdsin, rfdsout;
  char buffer [BSIZE+1];
  int bufitr = 0;
  struct termios my_termios;

  int i;
  int j;
  int packetcntr = 0;
  int ifprint = 1; /*0 if it hsould print*/
  unsigned int first;
  unsigned int second;//used for getting data
  int fndx; //used to get first char
  int sndx; //used to get second char
  int cindex; //index to cross check if a packet is incomplete;
  FILE *fp;
  FILE *f2;
  unsigned int temp;
  double mydata;
  double myoutput;  
  double totalFC = 0;  

  fp = fopen ("raw.txt", "w");
  f2 = fopen ("processed.txt", "w");
  fd = open ("/dev/ttyS0", O_RDWR | O_NOCTTY);

  
   //configure serial port
  tcgetattr(fd, &my_termios);
  tcflush(fd, TCIFLUSH);
  my_termios.c_cflag = B9600 | CS8 | CREAD | CLOCAL | HUPCL;
  cfsetospeed (&my_termios, B9600);
  tcsetattr(fd, TCSANOW, &my_termios);

  /*print header*/
  printf ("\n\tTPS\tTIMER0\tTIMER1\tTEMP\tMAP\tA/F\n");
  
  /*read first potential packet*/
  for (bufitr = 0; bufitr<PSIZE; bufitr++) {
        read (fd, &buffer[bufitr], 1);	 
	fprintf (fp, "%c", buffer[bufitr]);
  }

  /*for all subsequent packets*/
  while (1) {
    read (fd, &buffer[bufitr], 1);	
    fprintf (fp, "%c", buffer[bufitr]);
 
    if (buffer[bufitr] == '\r') {
      /*set compare index*/
      if (bufitr < PSIZE)
	cindex = BSIZE+1 +bufitr - PSIZE;
      else
	cindex = bufitr - PSIZE;
      
      /*process if complete*/
      if (buffer[bufitr] == buffer[cindex]){
	fndx = cindex+5;
	/* process data - FIRST TWO CHANNELS IGNORED!*/
	for (j=2; j<DATINPACK; j++) {
	  /*check if near end of buffer*/
	  if (fndx<BSIZE)
	    sndx=fndx +1;
	  else if (fndx==BSIZE)
	    sndx = 0;
	  else if (fndx>BSIZE) {
	    fndx = 0;
	    sndx = 1;
	  }
		  
	  /*convert first char to hex*/
	  if (buffer[fndx] >= '0' && buffer[fndx] <= '9')
	    first = buffer[fndx] - 48;
	  else if (buffer[fndx] >= 'A' && buffer[fndx] <= 'F')
	    first = buffer[fndx] - 55;
	  
	  /*convert sec char to hex*/
	  if (buffer[sndx] >= '0' && buffer[sndx] <= '9')
	    second = buffer[sndx] - 48;
	  else if (buffer[sndx] >= 'A' && buffer[sndx] <= 'F')
	    second = buffer[sndx] - 55;
	  
	  first = first<<4;
	  first = first | second;
	  mydata = first*VPI;
	  
	  switch (j) {
	  case 2: 
	    myoutput = (mydata - TPSOFF) * TPSCONV * 100;
	    if (ifprint == 0)
		printf ("\t%.0f%%", myoutput);
	    break;
	  case 3:
	    myoutput = mydata * RPMCONV;
	    if (ifprint == 0)
	    	printf ("\t%.0f", myoutput);
	    break;
	  case 4:
	    myoutput = mydata * PRESCALER * FLOWRT / T1FREQ;
	    totalFC += myoutput;
	    if (ifprint == 0)
		printf ("\t%.3f", totalFC);
	    break;
	  case 5:
	    myoutput = mydata;
	    if (ifprint == 0)
		printf ("\t%.2f", myoutput);
	    break;
	  case 6:
	    myoutput = mydata * MAPCONV + MAPOFF;
	    if (ifprint == 0)
		printf ("\t%.2f", myoutput);
	    break;
	  case 7:
	    myoutput = mydata;
	    if (ifprint == 0)
		printf ("\t%.2f", myoutput);
	    break;
	  } /*end switch*/
	   
	  fprintf (f2, "%f ", myoutput);
	  
	  //iterate internal buffer iteraror
	  fndx +=2;
	  
	}/*end for*/
	fprintf(f2, "\n");
	printf("\r");

	packetcntr++;
	ifprint = packetcntr % OUTPUTCNTR;
      }/*end check for complete packet*/
    }/*continue reading*/
        
    /*iterate buffer*/
    if (bufitr == (BSIZE)) /*circle around if end of buffer*/
      bufitr = 0;
    else 
      bufitr++;
  }
  printf("bye\n");
  
  close (fd);
  fclose(fp);
}
